@@ -1,6 +1,6 @@
 /* Multi-process control for GDB, the GNU debugger.
 
-   Copyright (C) 2008, 2009, 2010 Free Software Foundation, Inc.
+   Copyright (C) 2008-2013 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -22,6 +22,7 @@
 #include "inferior.h"
 #include "target.h"
 #include "command.h"
+#include "completer.h"
 #include "gdbcmd.h"
 #include "gdbthread.h"
 #include "ui-out.h"
@@ -30,11 +31,17 @@
 #include "gdbcore.h"
 #include "symfile.h"
 #include "environ.h"
+#include "cli/cli-utils.h"
+#include "continuations.h"
+#include "arch-utils.h"
+#include "target-descriptions.h"
 
 void _initialize_inferiors (void);
 
-static void inferior_alloc_data (struct inferior *inf);
-static void inferior_free_data (struct inferior *inf);
+/* Keep a registry of per-inferior data-pointers required by other GDB
+   modules.  */
+
+DEFINE_REGISTRY (inferior, REGISTRY_ACCESS_FIELD)
 
 struct inferior *inferior_list = NULL;
 static int highest_inferior_num;
@@ -68,6 +75,7 @@ static void
 restore_inferior (void *arg)
 {
   struct inferior *saved_inferior = arg;
+
   set_current_inferior (saved_inferior);
 }
 
@@ -80,6 +88,7 @@ save_current_inferior (void)
 {
   struct cleanup *old_chain = make_cleanup (restore_inferior,
 					    current_inferior_);
+
   return old_chain;
 }
 
@@ -91,6 +100,7 @@ free_inferior (struct inferior *inf)
   xfree (inf->args);
   xfree (inf->terminal);
   free_environ (inf->environment);
+  target_desc_info_free (inf->tdesc_info);
   xfree (inf->private);
   xfree (inf);
 }
@@ -122,7 +132,7 @@ add_inferior_silent (int pid)
   memset (inf, 0, sizeof (*inf));
   inf->pid = pid;
 
-  inf->stop_soon = NO_STOP_QUIETLY;
+  inf->control.stop_soon = NO_STOP_QUIETLY;
 
   inf->num = ++highest_inferior_num;
   inf->next = inferior_list;
@@ -133,6 +143,8 @@ add_inferior_silent (int pid)
 
   inferior_alloc_data (inf);
 
+  observer_notify_inferior_added (inf);
+
   if (pid != 0)
     inferior_appeared (inf, pid);
 
@@ -172,29 +184,10 @@ delete_thread_of_inferior (struct thread_info *tp, void *data)
   return 0;
 }
 
-void
-delete_threads_of_inferior (int pid)
-{
-  struct inferior *inf;
-  struct delete_thread_of_inferior_arg arg;
-
-  for (inf = inferior_list; inf; inf = inf->next)
-    if (inf->pid == pid)
-      break;
-
-  if (!inf)
-    return;
-
-  arg.pid = pid;
-  arg.silent = 1;
-
-  iterate_over_threads (delete_thread_of_inferior, &arg);
-}
-
 /* If SILENT then be quiet -- don't announce a inferior death, or the
    exit of its threads.  */
 
-static void
+void
 delete_inferior_1 (struct inferior *todel, int silent)
 {
   struct inferior *inf, *infprev;
@@ -219,6 +212,8 @@ delete_inferior_1 (struct inferior *todel, int silent)
   else
     inferior_list = inf->next;
 
+  observer_notify_inferior_removed (inf);
+
   free_inferior (inf);
 }
 
@@ -265,20 +260,31 @@ exit_inferior_1 (struct inferior *inftoex, int silent)
 
   /* Notify the observers before removing the inferior from the list,
      so that the observers have a chance to look it up.  */
-  observer_notify_inferior_exit (inf->pid);
+  observer_notify_inferior_exit (inf);
 
   inf->pid = 0;
+  inf->fake_pid_p = 0;
   if (inf->vfork_parent != NULL)
     {
       inf->vfork_parent->vfork_child = NULL;
       inf->vfork_parent = NULL;
     }
+  if (inf->vfork_child != NULL)
+    {
+      inf->vfork_child->vfork_parent = NULL;
+      inf->vfork_child = NULL;
+    }
+
+  inf->has_exit_code = 0;
+  inf->exit_code = 0;
+  inf->pending_detach = 0;
 }
 
 void
 exit_inferior (int pid)
 {
   struct inferior *inf = find_inferior_pid (pid);
+
   exit_inferior_1 (inf, 0);
 
   if (print_inferior_events)
@@ -289,6 +295,7 @@ void
 exit_inferior_silent (int pid)
 {
   struct inferior *inf = find_inferior_pid (pid);
+
   exit_inferior_1 (inf, 1);
 }
 
@@ -304,6 +311,7 @@ void
 detach_inferior (int pid)
 {
   struct inferior *inf = find_inferior_pid (pid);
+
   exit_inferior_1 (inf, 1);
 
   if (print_inferior_events)
@@ -315,7 +323,7 @@ inferior_appeared (struct inferior *inf, int pid)
 {
   inf->pid = pid;
 
-  observer_notify_inferior_appeared (pid);
+  observer_notify_inferior_appeared (inf);
 }
 
 void
@@ -452,16 +460,19 @@ have_inferiors (void)
 int
 have_live_inferiors (void)
 {
-  struct target_ops *t;
+  struct inferior *inf;
 
-  /* The check on stratum suffices, as GDB doesn't currently support
-     multiple target interfaces.  */
-  if (have_inferiors ())
-    for (t = current_target.beneath; t != NULL; t = t->beneath)
-      if (t->to_stratum == process_stratum)
-	return 1;
+  for (inf = inferior_list; inf; inf = inf->next)
+    if (inf->pid != 0)
+      {
+	struct thread_info *tp;
+	
+	tp = any_thread_of_process (inf->pid);
+	if (tp && target_has_execution_1 (tp->ptid))
+	  break;
+      }
 
-  return 0;
+  return inf != NULL;
 }
 
 /* Prune away automatically added program spaces that aren't required
@@ -508,13 +519,27 @@ number_of_inferiors (void)
   return count;
 }
 
+/* Converts an inferior process id to a string.  Like
+   target_pid_to_str, but special cases the null process.  */
+
+static char *
+inferior_pid_to_str (int pid)
+{
+  if (pid != 0)
+    return target_pid_to_str (pid_to_ptid (pid));
+  else
+    return _("<null>");
+}
+
 /* Prints the list of inferiors and their details on UIOUT.  This is a
    version of 'info_inferior_command' suitable for use from MI.
 
-   If REQUESTED_INFERIOR is not -1, it's the GDB id of the inferior that
-   should be printed.  Otherwise, all inferiors are printed.  */
-void
-print_inferior (struct ui_out *uiout, int requested_inferior)
+   If REQUESTED_INFERIORS is not NULL, it's a list of GDB ids of the
+   inferiors that should be printed.  Otherwise, all inferiors are
+   printed.  */
+
+static void
+print_inferior (struct ui_out *uiout, char *requested_inferiors)
 {
   struct inferior *inf;
   struct cleanup *old_chain;
@@ -523,9 +548,7 @@ print_inferior (struct ui_out *uiout, int requested_inferior)
   /* Compute number of inferiors we will print.  */
   for (inf = inferior_list; inf; inf = inf->next)
     {
-      struct cleanup *chain2;
-
-      if (requested_inferior != -1 && inf->num != requested_inferior)
+      if (!number_is_in_list (requested_inferiors, inf->num))
 	continue;
 
       ++inf_count;
@@ -549,7 +572,7 @@ print_inferior (struct ui_out *uiout, int requested_inferior)
     {
       struct cleanup *chain2;
 
-      if (requested_inferior != -1 && inf->num != requested_inferior)
+      if (!number_is_in_list (requested_inferiors, inf->num))
 	continue;
 
       chain2 = make_cleanup_ui_out_tuple_begin_end (uiout, NULL);
@@ -561,15 +584,11 @@ print_inferior (struct ui_out *uiout, int requested_inferior)
 
       ui_out_field_int (uiout, "number", inf->num);
 
-      if (inf->pid)
-	ui_out_field_string (uiout, "target-id",
-			     target_pid_to_str (pid_to_ptid (inf->pid)));
-      else
-	ui_out_field_string (uiout, "target-id", "<null>");
+      ui_out_field_string (uiout, "target-id",
+			   inferior_pid_to_str (inf->pid));
 
-      if (inf->pspace->ebfd)
-	ui_out_field_string (uiout, "exec",
-			     bfd_get_filename (inf->pspace->ebfd));
+      if (inf->pspace->pspace_exec_filename != NULL)
+	ui_out_field_string (uiout, "exec", inf->pspace->pspace_exec_filename);
       else
 	ui_out_field_skip (uiout, "exec");
 
@@ -599,24 +618,35 @@ detach_inferior_command (char *args, int from_tty)
 {
   int num, pid;
   struct thread_info *tp;
+  struct get_number_or_range_state state;
 
   if (!args || !*args)
-    error (_("Requires argument (inferior id to detach)"));
+    error (_("Requires argument (inferior id(s) to detach)"));
 
-  num = parse_and_eval_long (args);
+  init_number_or_range (&state, args);
+  while (!state.finished)
+    {
+      num = get_number_or_range (&state);
 
-  if (!valid_gdb_inferior_id (num))
-    error (_("Inferior ID %d not known."), num);
+      if (!valid_gdb_inferior_id (num))
+	{
+	  warning (_("Inferior ID %d not known."), num);
+	  continue;
+	}
 
-  pid = gdb_inferior_id_to_pid (num);
+      pid = gdb_inferior_id_to_pid (num);
 
-  tp = any_thread_of_process (pid);
-  if (!tp)
-    error (_("Inferior has no threads."));
+      tp = any_thread_of_process (pid);
+      if (!tp)
+	{
+	  warning (_("Inferior ID %d has no threads."), num);
+	  continue;
+	}
 
-  switch_to_thread (tp->ptid);
+      switch_to_thread (tp->ptid);
 
-  detach_command (NULL, from_tty);
+      detach_command (NULL, from_tty);
+    }
 }
 
 static void
@@ -624,24 +654,35 @@ kill_inferior_command (char *args, int from_tty)
 {
   int num, pid;
   struct thread_info *tp;
+  struct get_number_or_range_state state;
 
   if (!args || !*args)
-    error (_("Requires argument (inferior id to kill)"));
+    error (_("Requires argument (inferior id(s) to kill)"));
 
-  num = parse_and_eval_long (args);
+  init_number_or_range (&state, args);
+  while (!state.finished)
+    {
+      num = get_number_or_range (&state);
 
-  if (!valid_gdb_inferior_id (num))
-    error (_("Inferior ID %d not known."), num);
+      if (!valid_gdb_inferior_id (num))
+	{
+	  warning (_("Inferior ID %d not known."), num);
+	  continue;
+	}
 
-  pid = gdb_inferior_id_to_pid (num);
+      pid = gdb_inferior_id_to_pid (num);
 
-  tp = any_thread_of_process (pid);
-  if (!tp)
-    error (_("Inferior has no threads."));
+      tp = any_thread_of_process (pid);
+      if (!tp)
+	{
+	  warning (_("Inferior ID %d has no threads."), num);
+	  continue;
+	}
 
-  switch_to_thread (tp->ptid);
+      switch_to_thread (tp->ptid);
 
-  target_kill ();
+      target_kill ();
+    }
 
   bfd_cache_close_all ();
 }
@@ -660,9 +701,9 @@ inferior_command (char *args, int from_tty)
 
   printf_filtered (_("[Switching to inferior %d [%s] (%s)]\n"),
 		   inf->num,
-		   target_pid_to_str (pid_to_ptid (inf->pid)),
-		   (inf->pspace->ebfd
-		    ? bfd_get_filename (inf->pspace->ebfd)
+		   inferior_pid_to_str (inf->pid),
+		   (inf->pspace->pspace_exec_filename != NULL
+		    ? inf->pspace->pspace_exec_filename
 		    : _("<noexec>")));
 
   if (inf->pid != 0)
@@ -693,10 +734,10 @@ inferior_command (char *args, int from_tty)
     }
 
   if (inf->pid != 0 && is_running (inferior_ptid))
-    ui_out_text (uiout, "(running)\n");
+    ui_out_text (current_uiout, "(running)\n");
   else if (inf->pid != 0)
     {
-      ui_out_text (uiout, "\n");
+      ui_out_text (current_uiout, "\n");
       print_stack_frame (get_selected_frame (NULL), 1, SRC_AND_LOC);
     }
 }
@@ -706,42 +747,80 @@ inferior_command (char *args, int from_tty)
 static void
 info_inferiors_command (char *args, int from_tty)
 {
-  int requested = -1;
-
-  if (args && *args)
-    {
-      requested = parse_and_eval_long (args);
-      if (!valid_gdb_inferior_id (requested))
-	error (_("Inferior ID %d not known."), requested);
-    }
-
-  print_inferior (uiout, requested);
+  print_inferior (current_uiout, args);
 }
 
 /* remove-inferior ID */
 
-void
+static void
 remove_inferior_command (char *args, int from_tty)
 {
   int num;
   struct inferior *inf;
+  struct get_number_or_range_state state;
 
-  num = parse_and_eval_long (args);
-  inf = find_inferior_id (num);
+  if (args == NULL || *args == '\0')
+    error (_("Requires an argument (inferior id(s) to remove)"));
 
-  if (inf == NULL)
-    error (_("Inferior ID %d not known."), num);
+  init_number_or_range (&state, args);
+  while (!state.finished)
+    {
+      num = get_number_or_range (&state);
+      inf = find_inferior_id (num);
 
-  if (inf == current_inferior ())
-    error (_("Can not remove current symbol inferior."));
+      if (inf == NULL)
+	{
+	  warning (_("Inferior ID %d not known."), num);
+	  continue;
+	}
 
-  delete_inferior_1 (inf, 1);
+      if (inf == current_inferior ())
+	{
+	  warning (_("Can not remove current symbol inferior %d."), num);
+	  continue;
+	}
+    
+      if (inf->pid != 0)
+	{
+	  warning (_("Can not remove active inferior %d."), num);
+	  continue;
+	}
+
+      delete_inferior_1 (inf, 1);
+    }
 }
 
+struct inferior *
+add_inferior_with_spaces (void)
+{
+  struct address_space *aspace;
+  struct program_space *pspace;
+  struct inferior *inf;
+  struct gdbarch_info info;
+
+  /* If all inferiors share an address space on this system, this
+     doesn't really return a new address space; otherwise, it
+     really does.  */
+  aspace = maybe_new_address_space ();
+  pspace = add_program_space (aspace);
+  inf = add_inferior (0);
+  inf->pspace = pspace;
+  inf->aspace = pspace->aspace;
+
+  /* Setup the inferior's initial arch, based on information obtained
+     from the global "set ..." options.  */
+  gdbarch_info_init (&info);
+  inf->gdbarch = gdbarch_find_by_info (info);
+  /* The "set ..." options reject invalid settings, so we should
+     always have a valid arch by now.  */
+  gdb_assert (inf->gdbarch != NULL);
+
+  return inf;
+}
 
 /* add-inferior [-copies N] [-exec FILENAME]  */
 
-void
+static void
 add_inferior_command (char *args, int from_tty)
 {
   int i, copies = 1;
@@ -782,26 +861,15 @@ add_inferior_command (char *args, int from_tty)
 
   for (i = 0; i < copies; ++i)
     {
-      struct address_space *aspace;
-      struct program_space *pspace;
-      struct inferior *inf;
-
-      /* If all inferiors share an address space on this system, this
-	 doesn't really return a new address space; otherwise, it
-	 really does.  */
-      aspace = maybe_new_address_space ();
-      pspace = add_program_space (aspace);
-      inf = add_inferior (0);
-      inf->pspace = pspace;
-      inf->aspace = pspace->aspace;
+      struct inferior *inf = add_inferior_with_spaces ();
 
       printf_filtered (_("Added inferior %d\n"), inf->num);
 
       if (exec != NULL)
 	{
 	  /* Switch over temporarily, while reading executable and
-	     symbols.q  */
-	  set_current_program_space (pspace);
+	     symbols.q.  */
+	  set_current_program_space (inf->pspace);
 	  set_current_inferior (inf);
 	  switch_to_thread (null_ptid);
 
@@ -815,7 +883,7 @@ add_inferior_command (char *args, int from_tty)
 
 /* clone-inferior [-copies N] [ID] */
 
-void
+static void
 clone_inferior_command (char *args, int from_tty)
 {
   int i, copies = 1;
@@ -885,6 +953,12 @@ clone_inferior_command (char *args, int from_tty)
       inf = add_inferior (0);
       inf->pspace = pspace;
       inf->aspace = pspace->aspace;
+      inf->gdbarch = orginf->gdbarch;
+
+      /* If the original inferior had a user specified target
+	 description, make the clone use it too.  */
+      if (target_desc_info_from_user_p (inf->tdesc_info))
+	copy_inferior_target_desc_info (inf, orginf);
 
       printf_filtered (_("Added inferior %d.\n"), inf->num);
 
@@ -906,108 +980,11 @@ show_print_inferior_events (struct ui_file *file, int from_tty,
 
 
 
-/* Keep a registry of per-inferior data-pointers required by other GDB
-   modules.  */
-
-struct inferior_data
-{
-  unsigned index;
-  void (*cleanup) (struct inferior *, void *);
-};
-
-struct inferior_data_registration
-{
-  struct inferior_data *data;
-  struct inferior_data_registration *next;
-};
-
-struct inferior_data_registry
-{
-  struct inferior_data_registration *registrations;
-  unsigned num_registrations;
-};
-
-static struct inferior_data_registry inferior_data_registry
-  = { NULL, 0 };
-
-const struct inferior_data *
-register_inferior_data_with_cleanup
-  (void (*cleanup) (struct inferior *, void *))
-{
-  struct inferior_data_registration **curr;
-
-  /* Append new registration.  */
-  for (curr = &inferior_data_registry.registrations;
-       *curr != NULL; curr = &(*curr)->next);
-
-  *curr = XMALLOC (struct inferior_data_registration);
-  (*curr)->next = NULL;
-  (*curr)->data = XMALLOC (struct inferior_data);
-  (*curr)->data->index = inferior_data_registry.num_registrations++;
-  (*curr)->data->cleanup = cleanup;
-
-  return (*curr)->data;
-}
-
-const struct inferior_data *
-register_inferior_data (void)
-{
-  return register_inferior_data_with_cleanup (NULL);
-}
-
-static void
-inferior_alloc_data (struct inferior *inf)
-{
-  gdb_assert (inf->data == NULL);
-  inf->num_data = inferior_data_registry.num_registrations;
-  inf->data = XCALLOC (inf->num_data, void *);
-}
-
-static void
-inferior_free_data (struct inferior *inf)
-{
-  gdb_assert (inf->data != NULL);
-  clear_inferior_data (inf);
-  xfree (inf->data);
-  inf->data = NULL;
-}
-
-void
-clear_inferior_data (struct inferior *inf)
-{
-  struct inferior_data_registration *registration;
-  int i;
-
-  gdb_assert (inf->data != NULL);
-
-  for (registration = inferior_data_registry.registrations, i = 0;
-       i < inf->num_data;
-       registration = registration->next, i++)
-    if (inf->data[i] != NULL && registration->data->cleanup)
-      registration->data->cleanup (inf, inf->data[i]);
-
-  memset (inf->data, 0, inf->num_data * sizeof (void *));
-}
-
-void
-set_inferior_data (struct inferior *inf,
-		       const struct inferior_data *data,
-		       void *value)
-{
-  gdb_assert (data->index < inf->num_data);
-  inf->data[data->index] = value;
-}
-
-void *
-inferior_data (struct inferior *inf, const struct inferior_data *data)
-{
-  gdb_assert (data->index < inf->num_data);
-  return inf->data[data->index];
-}
-
 void
 initialize_inferiors (void)
 {
+  struct cmd_list_element *c = NULL;
+
   /* There's always one inferior.  Note that this function isn't an
      automatic _initialize_foo function, since other _initialize_foo
      routines may need to install their per-inferior data keys.  We
@@ -1017,20 +994,23 @@ initialize_inferiors (void)
   current_inferior_ = add_inferior (0);
   current_inferior_->pspace = current_program_space;
   current_inferior_->aspace = current_program_space->aspace;
+  /* The architecture will be initialized shortly, by
+     initialize_current_architecture.  */
 
-  add_info ("inferiors", info_inferiors_command,
-	    _("IDs of currently known inferiors."));
+  add_info ("inferiors", info_inferiors_command, 
+	    _("IDs of specified inferiors (all inferiors if no argument)."));
 
-  add_com ("add-inferior", no_class, add_inferior_command, _("\
+  c = add_com ("add-inferior", no_class, add_inferior_command, _("\
 Add a new inferior.\n\
 Usage: add-inferior [-copies <N>] [-exec <FILENAME>]\n\
-N is the optional number of inferior to add, default is 1.\n\
+N is the optional number of inferiors to add, default is 1.\n\
 FILENAME is the file name of the executable to use\n\
 as main program."));
+  set_cmd_completer (c, filename_completer);
 
-  add_com ("remove-inferior", no_class, remove_inferior_command, _("\
-Remove inferior ID.\n\
-Usage: remove-inferior ID"));
+  add_com ("remove-inferiors", no_class, remove_inferior_command, _("\
+Remove inferior ID (or list of IDs).\n\
+Usage: remove-inferiors ID..."));
 
   add_com ("clone-inferior", no_class, clone_inferior_command, _("\
 Clone inferior ID.\n\
@@ -1040,12 +1020,12 @@ executable loaded as the copied inferior.  If -copies is not specified,\n\
 adds 1 copy.  If ID is not specified, it is the current inferior\n\
 that is cloned."));
 
-  add_cmd ("inferior", class_run, detach_inferior_command, _("\
-Detach from inferior ID."),
+  add_cmd ("inferiors", class_run, detach_inferior_command, _("\
+Detach from inferior ID (or list of IDS)."),
 	   &detachlist);
 
-  add_cmd ("inferior", class_run, kill_inferior_command, _("\
-Kill inferior ID."),
+  add_cmd ("inferiors", class_run, kill_inferior_command, _("\
+Kill inferior ID (or list of IDs)."),
 	   &killlist);
 
   add_cmd ("inferior", class_run, inferior_command, _("\
